﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace CanonicalSplineDemo
{
    public partial class MainPage : UserControl
    {
        Dot draggedDot;
        Point startPointMouse;
        Point startPointDot;

        public MainPage()
        {
            InitializeComponent();

            PointCollection points = new PointCollection();
            spline.Points = points;
            SynchronizeDotsWithPoints();

            tensionScroll.Value = spline.Tension;
            tensionText.Text = spline.Tension.ToString("F1");
        }

        void OnScrollValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            spline.Tension = args.NewValue;
            tensionText.Text = args.NewValue.ToString("F1");
        }

        void OnCloseSplineChanged(object sender, RoutedEventArgs args)
        {
            spline.IsClosed = (bool)(sender as CheckBox).IsChecked;
        }

        void OnFillChanged(object sender, RoutedEventArgs args)
        {
            radioGroup.IsEnabled = (bool)(sender as CheckBox).IsChecked;
            spline.IsFilled = radioGroup.IsEnabled;
            spline.Fill = radioGroup.IsEnabled ? new SolidColorBrush(Colors.Cyan) : null;
        }

        void OnFillRuleChecked(object sender, RoutedEventArgs args)
        {
            if (spline != null)
                spline.FillRule = (bool)radioEvenOdd.IsChecked ? FillRule.EvenOdd : FillRule.Nonzero;
        }

        void OnAddPoint(object sender, RoutedEventArgs args)
        {
            Dot dot = new Dot();
            int num = canvas.Children.Count;
            dot.Center = new Point(num * 25, num * 25);
            canvas.Children.Add(dot);
            buttonRemove.IsEnabled = true;

            SynchronizeDotsWithPoints();
        }

        void OnRemovePoint(object sender, RoutedEventArgs args)
        {
            if (canvas.Children.Count > 1)
                canvas.Children.RemoveAt(canvas.Children.Count - 1);

            if (canvas.Children.Count == 1)
                buttonRemove.IsEnabled = false;

            SynchronizeDotsWithPoints();
        }

        void SynchronizeDotsWithPoints()
        {
            PointCollection points = spline.Points;
            spline.Points = null;
            points.Clear();

            foreach (UIElement child in canvas.Children)
            {
                Dot dot = child as Dot;
                if (dot != null)
                    points.Add(dot.Center);
            }

            spline.Points = points;
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonDown(args);

            if (args.OriginalSource is Path)
            {
                DependencyObject parent = (args.OriginalSource as Path).Parent;

                if (parent is Dot)
                {
                    draggedDot = parent as Dot;
                    startPointDot = draggedDot.Center;
                    startPointMouse = args.GetPosition(this);
                    CaptureMouse();
                }
            }
        }

        protected override void OnMouseMove(MouseEventArgs args)
        {
            base.OnMouseMove(args);

            if (draggedDot != null)
            {
                Point pointMouse = args.GetPosition(this);
                draggedDot.Center = new Point(startPointDot.X - startPointMouse.X + pointMouse.X,
                                              startPointDot.Y - startPointMouse.Y + pointMouse.Y);

                SynchronizeDotsWithPoints();
            }
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonUp(args);

            if (draggedDot != null)
            {
                ReleaseMouseCapture();
                draggedDot = null;
            }
        }
    }
}
